<?php

defined( 'ABSPATH' ) || exit;

/**
*
* Returns whether the product is bookable or not.
* @param mixed int | WC_Product or WC_Product_Variation - $product
* @return bool
*
**/
function wceb_is_bookable( $_product ) {

    // If product ID was passed, get the product
    if ( is_numeric( $_product ) ) {
        $_product = wc_get_product( $_product );
    }

    if ( ! $_product ) {
        return false;
    }

    $product_type = $_product->get_type();

    // Check the product type
    $allowed_product_types   = wceb_get_allowed_product_types();
    $allowed_product_types[] = 'variation';

    $is_bookable = $_product->get_meta( '_bookable', true );

    // Backward compatibility < 2.2.4
    if ( empty( $is_bookable ) ) {
        $is_bookable = $_product->get_meta( '_booking_option', true );
    }

    // Check that variable products have at least one bookable variation.
    if ( $product_type === 'variable' ) {

        $has_bookable_variation = false;
        foreach ( $_product->get_children() as $variation_id ) {

            $variation = wc_get_product( $variation_id );

            if ( wceb_is_bookable( $variation ) ) {
                $has_bookable_variation = true;
                break;
            }

        }

        if ( ! $has_bookable_variation ) {
            return false;
        }

    }

    $is_bookable = ( $is_bookable === 'yes' && in_array( $product_type, $allowed_product_types ) ) ? true : false;

    return apply_filters( 'easy_booking_product_is_bookable', $is_bookable, $_product );

}

/**
*
* Get product booking settings.
*
* @param WC_Product or WC_Product_Variation - $_product
* @return array - $booking_settings
*
**/
function wceb_get_product_booking_settings( $_product ) {

    $booking_settings = array(
        'booking_dates'           => wceb_get_product_number_of_dates_to_select( $_product ),
        'booking_duration'        => wceb_get_product_booking_duration( $_product ),
        'custom_booking_duration' => wceb_get_product_custom_booking_duration( $_product ),
        'booking_min'             => wceb_get_product_minimum_booking_duration( $_product ),
        'booking_max'             => wceb_get_product_maximum_booking_duration( $_product ),
        'first_available_date'    => wceb_get_product_first_available_date( $_product )
    );

    return $booking_settings;

}

/**
*
* Get product minimum booking duration.
* @param WC_Product or WC_Product_Variation - $_product
* @return int - $booking_min
*
**/
function wceb_get_product_minimum_booking_duration( $_product ) {

    // If product setting is empty or not defined, get global setting
    $global_booking_min  = get_option( 'wceb_booking_min' );
    $product_booking_min = $_product->get_meta( '_booking_min', true );

    if ( $_product->is_type( 'variation' ) && ( ! isset( $product_booking_min ) || $product_booking_min === '' ) ) {

        $parent_product_id = $_product->get_parent_id();
        $parent_product = wc_get_product( $parent_product_id );

        $product_booking_min = $parent_product->get_meta( '_booking_min', true );

    }

    $booking_min = isset( $product_booking_min ) && $product_booking_min !== '' ? $product_booking_min : $global_booking_min;

    // Get booking mode - Days or Nights
    $booking_mode = get_option( 'wceb_booking_mode' );

    // Get custom booking duration
    $custom_booking_duration = wceb_get_product_custom_booking_duration( $_product );

    // Multiply by custom booking duration (remove 1 day in "Days" mode)
    $booking_min = $booking_mode === 'days' ? $booking_min * $custom_booking_duration - 1 : $booking_min * $custom_booking_duration;

    if ( $booking_min < 0 ) {
        $booking_min = 0;
    }

    return apply_filters( 'easy_booking_product_minimum_booking_duration', $booking_min, $_product );

}

/**
*
* Get product maximum booking duration.
* @param WC_Product or WC_Product_Variation - $_product
* @return int - $booking_max
*
**/
function wceb_get_product_maximum_booking_duration( $_product ) {

    // If product setting is empty or not defined, get global setting
    $global_booking_max  = get_option( 'wceb_booking_max' );
    $product_booking_max = $_product->get_meta( '_booking_max', true );

    if ( $_product->is_type( 'variation' ) && ( ! isset( $product_booking_max ) || $product_booking_max === '' ) ) {

        $parent_product_id = $_product->get_parent_id();
        $parent_product = wc_get_product( $parent_product_id );

        $product_booking_max = $parent_product->get_meta( '_booking_max', true );

    }
    
    $booking_max = isset( $product_booking_max ) && $product_booking_max !== '' ? $product_booking_max : $global_booking_max;

    // Get booking mode - Days or Nights
    $booking_mode = get_option( 'wceb_booking_mode' );

    // Get custom booking duration
    $custom_booking_duration = wceb_get_product_custom_booking_duration( $_product );

    // Multiply by custom booking duration (remove 1 day in "Days" mode)
    $booking_max = $booking_mode === 'days' ? $booking_max * $custom_booking_duration - 1 : $booking_max * $custom_booking_duration;

    if ( $booking_max <= 0 ) {
        $booking_max = false;
    }

    return apply_filters( 'easy_booking_product_maximum_booking_duration', $booking_max, $_product );

}

/**
*
* Get product first available date.
* @param WC_Product or WC_Product_Variation - $_product
* @return int - $first_available_date
*
**/
function wceb_get_product_first_available_date( $_product ) {

    // If product setting is empty or not defined, get global setting
    $global_first_available_date  = get_option( 'wceb_first_available_date' );
    $product_first_available_date = $_product->get_meta( '_first_available_date', true );

    if ( $_product->is_type( 'variation' ) && ( ! isset( $product_first_available_date ) || $product_first_available_date === '' ) ) {

        $parent_product_id = $_product->get_parent_id();
        $parent_product = wc_get_product( $parent_product_id );

        $product_first_available_date = $parent_product->get_meta( '_first_available_date', true );

    }
    
    $first_available_date = isset( $product_first_available_date ) && $product_first_available_date !== '' ? $product_first_available_date : $global_first_available_date;

    return apply_filters( 'easy_booking_product_first_available_date', $first_available_date, $_product );

}

/**
*
* Get product booking duration.
*
* @param WC_Product or WC_Product_Variation - $_product
* @return str - $booking_duration
*
**/
function wceb_get_product_booking_duration( $_product ) {
    
    $booking_duration = $_product->get_meta( '_booking_duration', true );

    // For grouped products and bundles single product pages, force the parent product value.
    if ( is_product() ) {

        // Get queried product ID. For grouped and bundled products, it will return the parent product ID.
        $current_id = get_queried_object_id();
        $queried_product = wc_get_product( $current_id );
        
        // If it is a children or bundled product on the parent product page, return the parent data.
        if ( ( $queried_product->is_type( 'grouped' ) || $queried_product->is_type( 'bundle' ) ) && $current_id !== $_product->get_id() ) {
            $booking_duration = $queried_product->get_meta( '_booking_duration', true );
        }

    }

    // If it is a variation with the parent product settings.
    if ( $_product->is_type( 'variation' ) && ( $booking_duration === 'parent' || empty( $booking_duration ) ) ) {

        // Get parent product
        $parent_product_id = $_product->get_parent_id();
        $parent_product = wc_get_product( $parent_product_id );

        $booking_duration = $parent_product->get_meta( '_booking_duration', true );

    }

    if ( empty( $booking_duration ) || $booking_duration === 'global' ) {
        $booking_duration = get_option( 'wceb_booking_duration' );
    }

    return apply_filters( 'easy_booking_product_booking_duration', $booking_duration, $_product );

}

/**
*
* Get product custom booking duration.
*
* @param C_Product or WC_Product_Variation - $_product
* @return int - $custom_booking_duration
*
**/
function wceb_get_product_custom_booking_duration( $_product ) {

    $custom_booking_duration = $_product->get_meta( '_custom_booking_duration', true );

    // For grouped products and bundles single product pages, force the parent product value.
    if ( is_product() ) {

        // Get queried product ID. For grouped and bundled products, it will return the parent product ID.
        $current_id = get_queried_object_id();
        $queried_product = wc_get_product( $current_id );

        // If it is a children or bundled product on the parent product page, return the parent data.
        if ( ( $queried_product->is_type( 'grouped' ) || $queried_product->is_type( 'bundle' ) ) && $current_id !== $_product->get_id() ) {
            $custom_booking_duration = $queried_product->get_meta( '_custom_booking_duration', true );
        }

    }

    // If it is a variation with the parent product settings.
    if ( $_product->is_type( 'variation' ) && ( empty( $custom_booking_duration ) ) ) {
        
        // Get parent product
        $parent_product_id = $_product->get_parent_id();
        $parent_product = wc_get_product( $parent_product_id );

        $custom_booking_duration = $parent_product->get_meta( '_custom_booking_duration', true );

    }

    if ( empty( $custom_booking_duration ) ) {
        $custom_booking_duration = get_option( 'wceb_custom_booking_duration' );
    }

    $booking_duration = wceb_get_product_booking_duration( $_product );
    
    // Force custom duration if booking duration is set to "Weekly" (6 days or 7 nights)
    if ( $booking_duration === 'weeks' ) {

        $calc_mode = get_option( 'wceb_booking_mode' );
        $custom_booking_duration = $calc_mode === 'days' ? 6 : 7;

    }

    // Force custom durationif booking duration is set to "Daily" (1 day/night)
    if ( $booking_duration === 'days' ) {
        $custom_booking_duration = 1;
    }

    if ( $custom_booking_duration <= 0 ) {
        $custom_booking_duration = 1;
    }

    return apply_filters( 'easy_booking_product_custom_booking_duration', $custom_booking_duration, $_product );

}

/**
*
* Gets number of dates to select for the product.
*
* @param WC_Product or WC_Product_Variation - $_product
* @return str - $number_of_dates
*
**/
function wceb_get_product_number_of_dates_to_select( $_product ) {

    $number_of_dates = $_product->get_meta( '_number_of_dates', true );

    // Backward compatibility 2.2.5
    if ( empty( $number_of_dates ) ) {
        $number_of_dates = $_product->get_meta( '_booking_dates', true );
    }

    // For grouped products and bundles single product pages, force the parent product value.
    if ( is_product() ) {

    	// Get queried product ID. For grouped and bundled products, it will return the parent product ID.
    	$queried_id = get_queried_object_id();
    	$queried_product = wc_get_product( $queried_id );
        
    	// If it is a children or bundled product on the parent product page, return the parent data.
	    if ( ( $queried_product->is_type( 'grouped' ) || $queried_product->is_type( 'bundle' ) ) && $queried_id !== $_product->get_id() ) {

            $number_of_dates = $queried_product->get_meta( '_number_of_dates', true );

            // Backward compatibility 2.2.5
            if ( empty( $number_of_dates ) ) {
                $number_of_dates = $queried_product->get_meta( '_booking_dates', true );
            }

	    }
	    
	}

    // If it is a variation with the parent product settings.
    if ( $_product->is_type( 'variation' ) && ( $number_of_dates === 'parent' || empty( $number_of_dates ) ) ) {

        // Get parent product
        $parent_product_id = $_product->get_parent_id();
        $parent_product = wc_get_product( $parent_product_id );

        if ( $parent_product ) {

            $number_of_dates = $parent_product->get_meta( '_number_of_dates', true );

            // Backward compatibility 2.2.5
            if ( empty( $number_of_dates ) ) {
                $number_of_dates = $parent_product->get_meta( '_booking_dates', true );
            }

        }

    }

    if ( empty( $number_of_dates ) || $number_of_dates === 'global' ) {
    	$number_of_dates = get_option( 'wceb_number_of_dates' );
    }

    return apply_filters( 'easy_booking_product_number_of_dates_to_select', $number_of_dates, $_product );

}

/**
*
* Gets the price suffix for bookable products (e.g. "/ day").
*
* @param WC_Product or WC_Product_Variation - $_product
* @return str
*
**/
function wceb_get_product_price_suffix( $_product ) {

    $suffix = '';

    $number_of_dates  = wceb_get_product_number_of_dates_to_select( $_product );
    $booking_duration = wceb_get_product_booking_duration( $_product );
    $custom_duration  = wceb_get_product_custom_booking_duration( $_product );
    
    // For variable products, price html will be displayed in Javascript for each variation
    if ( $number_of_dates === 'two' && ! $_product->is_type( 'variable' ) ) {

        if ( $booking_duration === 'weeks' ) {

            $suffix = __( ' / week', 'woocommerce-easy-booking-system' );

        } else {

            $booking_mode = get_option( 'wceb_booking_mode' );

            $suffix = ( $booking_mode === 'nights' ) ? sprintf( _n( ' / night', ' / %s nights', $custom_duration, 'woocommerce-easy-booking-system' ), $custom_duration ) : sprintf( _n( ' / day', ' / %s days', $custom_duration, 'woocommerce-easy-booking-system' ), $custom_duration );

        }

    }

    // Backward compatibility
    $suffix = apply_filters_deprecated( 'easy_booking_get_price_html', array( $suffix, $_product, $booking_duration, $custom_duration ), '2.2.7', 'easy_booking_get_price_suffix' );

    return apply_filters( 'easy_booking_get_price_suffix', $suffix, $_product, $booking_duration, $custom_duration );

}